# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: messages.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='messages.proto',
  package='ZilliqaMessage',
  syntax='proto2',
  serialized_options=None,
  serialized_pb=_b('\n\x0emessages.proto\x12\x0eZilliqaMessage\"\x19\n\tByteArray\x12\x0c\n\x04\x64\x61ta\x18\x01 \x02(\x0c\"\x81\x02\n\x18ProtoTransactionCoreInfo\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\r\n\x05nonce\x18\x02 \x01(\x04\x12\x0e\n\x06toaddr\x18\x03 \x01(\x0c\x12/\n\x0csenderpubkey\x18\x04 \x01(\x0b\x32\x19.ZilliqaMessage.ByteArray\x12)\n\x06\x61mount\x18\x05 \x01(\x0b\x32\x19.ZilliqaMessage.ByteArray\x12+\n\x08gasprice\x18\x06 \x01(\x0b\x32\x19.ZilliqaMessage.ByteArray\x12\x10\n\x08gaslimit\x18\x07 \x01(\x04\x12\x0c\n\x04\x63ode\x18\x08 \x01(\x0c\x12\x0c\n\x04\x64\x61ta\x18\t \x01(\x0c\"\x88\x01\n\x10ProtoTransaction\x12\x0e\n\x06tranid\x18\x01 \x01(\x0c\x12\x36\n\x04info\x18\x02 \x01(\x0b\x32(.ZilliqaMessage.ProtoTransactionCoreInfo\x12,\n\tsignature\x18\x03 \x01(\x0b\x32\x19.ZilliqaMessage.ByteArray\":\n\x17ProtoTransactionReceipt\x12\x0f\n\x07receipt\x18\x01 \x01(\x0c\x12\x0e\n\x06\x63umgas\x18\x02 \x01(\x04\"\x8e\x01\n\x1bProtoTransactionWithReceipt\x12\x35\n\x0btransaction\x18\x01 \x01(\x0b\x32 .ZilliqaMessage.ProtoTransaction\x12\x38\n\x07receipt\x18\x02 \x01(\x0b\x32\'.ZilliqaMessage.ProtoTransactionReceipt')
)




_BYTEARRAY = _descriptor.Descriptor(
  name='ByteArray',
  full_name='ZilliqaMessage.ByteArray',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='ZilliqaMessage.ByteArray.data', index=0,
      number=1, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=34,
  serialized_end=59,
)


_PROTOTRANSACTIONCOREINFO = _descriptor.Descriptor(
  name='ProtoTransactionCoreInfo',
  full_name='ZilliqaMessage.ProtoTransactionCoreInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='ZilliqaMessage.ProtoTransactionCoreInfo.version', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nonce', full_name='ZilliqaMessage.ProtoTransactionCoreInfo.nonce', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='toaddr', full_name='ZilliqaMessage.ProtoTransactionCoreInfo.toaddr', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='senderpubkey', full_name='ZilliqaMessage.ProtoTransactionCoreInfo.senderpubkey', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='amount', full_name='ZilliqaMessage.ProtoTransactionCoreInfo.amount', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='gasprice', full_name='ZilliqaMessage.ProtoTransactionCoreInfo.gasprice', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='gaslimit', full_name='ZilliqaMessage.ProtoTransactionCoreInfo.gaslimit', index=6,
      number=7, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='code', full_name='ZilliqaMessage.ProtoTransactionCoreInfo.code', index=7,
      number=8, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='data', full_name='ZilliqaMessage.ProtoTransactionCoreInfo.data', index=8,
      number=9, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=62,
  serialized_end=319,
)


_PROTOTRANSACTION = _descriptor.Descriptor(
  name='ProtoTransaction',
  full_name='ZilliqaMessage.ProtoTransaction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tranid', full_name='ZilliqaMessage.ProtoTransaction.tranid', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='info', full_name='ZilliqaMessage.ProtoTransaction.info', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='signature', full_name='ZilliqaMessage.ProtoTransaction.signature', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=322,
  serialized_end=458,
)


_PROTOTRANSACTIONRECEIPT = _descriptor.Descriptor(
  name='ProtoTransactionReceipt',
  full_name='ZilliqaMessage.ProtoTransactionReceipt',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='receipt', full_name='ZilliqaMessage.ProtoTransactionReceipt.receipt', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cumgas', full_name='ZilliqaMessage.ProtoTransactionReceipt.cumgas', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=460,
  serialized_end=518,
)


_PROTOTRANSACTIONWITHRECEIPT = _descriptor.Descriptor(
  name='ProtoTransactionWithReceipt',
  full_name='ZilliqaMessage.ProtoTransactionWithReceipt',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='transaction', full_name='ZilliqaMessage.ProtoTransactionWithReceipt.transaction', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='receipt', full_name='ZilliqaMessage.ProtoTransactionWithReceipt.receipt', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=521,
  serialized_end=663,
)

_PROTOTRANSACTIONCOREINFO.fields_by_name['senderpubkey'].message_type = _BYTEARRAY
_PROTOTRANSACTIONCOREINFO.fields_by_name['amount'].message_type = _BYTEARRAY
_PROTOTRANSACTIONCOREINFO.fields_by_name['gasprice'].message_type = _BYTEARRAY
_PROTOTRANSACTION.fields_by_name['info'].message_type = _PROTOTRANSACTIONCOREINFO
_PROTOTRANSACTION.fields_by_name['signature'].message_type = _BYTEARRAY
_PROTOTRANSACTIONWITHRECEIPT.fields_by_name['transaction'].message_type = _PROTOTRANSACTION
_PROTOTRANSACTIONWITHRECEIPT.fields_by_name['receipt'].message_type = _PROTOTRANSACTIONRECEIPT
DESCRIPTOR.message_types_by_name['ByteArray'] = _BYTEARRAY
DESCRIPTOR.message_types_by_name['ProtoTransactionCoreInfo'] = _PROTOTRANSACTIONCOREINFO
DESCRIPTOR.message_types_by_name['ProtoTransaction'] = _PROTOTRANSACTION
DESCRIPTOR.message_types_by_name['ProtoTransactionReceipt'] = _PROTOTRANSACTIONRECEIPT
DESCRIPTOR.message_types_by_name['ProtoTransactionWithReceipt'] = _PROTOTRANSACTIONWITHRECEIPT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ByteArray = _reflection.GeneratedProtocolMessageType('ByteArray', (_message.Message,), dict(
  DESCRIPTOR = _BYTEARRAY,
  __module__ = 'messages_pb2'
  # @@protoc_insertion_point(class_scope:ZilliqaMessage.ByteArray)
  ))
_sym_db.RegisterMessage(ByteArray)

ProtoTransactionCoreInfo = _reflection.GeneratedProtocolMessageType('ProtoTransactionCoreInfo', (_message.Message,), dict(
  DESCRIPTOR = _PROTOTRANSACTIONCOREINFO,
  __module__ = 'messages_pb2'
  # @@protoc_insertion_point(class_scope:ZilliqaMessage.ProtoTransactionCoreInfo)
  ))
_sym_db.RegisterMessage(ProtoTransactionCoreInfo)

ProtoTransaction = _reflection.GeneratedProtocolMessageType('ProtoTransaction', (_message.Message,), dict(
  DESCRIPTOR = _PROTOTRANSACTION,
  __module__ = 'messages_pb2'
  # @@protoc_insertion_point(class_scope:ZilliqaMessage.ProtoTransaction)
  ))
_sym_db.RegisterMessage(ProtoTransaction)

ProtoTransactionReceipt = _reflection.GeneratedProtocolMessageType('ProtoTransactionReceipt', (_message.Message,), dict(
  DESCRIPTOR = _PROTOTRANSACTIONRECEIPT,
  __module__ = 'messages_pb2'
  # @@protoc_insertion_point(class_scope:ZilliqaMessage.ProtoTransactionReceipt)
  ))
_sym_db.RegisterMessage(ProtoTransactionReceipt)

ProtoTransactionWithReceipt = _reflection.GeneratedProtocolMessageType('ProtoTransactionWithReceipt', (_message.Message,), dict(
  DESCRIPTOR = _PROTOTRANSACTIONWITHRECEIPT,
  __module__ = 'messages_pb2'
  # @@protoc_insertion_point(class_scope:ZilliqaMessage.ProtoTransactionWithReceipt)
  ))
_sym_db.RegisterMessage(ProtoTransactionWithReceipt)


# @@protoc_insertion_point(module_scope)
